@@ -732,6 +732,30 @@ draw_string (GtkStyle * style,
     }
 }
 
+static void
+maybe_check_submenu_state (GtkMenuItem *menu_item, ThemeMatchData *match_data)
+{
+  /* Distinguish between active and passive focus, depending on whether the
+   * focus is in submenu.
+   *
+   * Active focus:
+   *   function = BOX
+   *   state    = PRELIGHT
+   *
+   * Passive focus:
+   *   function = BOX
+   *   state    = SELECTED
+   */
+  if (menu_item->submenu)
+    {
+      GtkWidget *sub_item;
+
+      sub_item = GTK_MENU_SHELL (menu_item->submenu)->active_menu_item;
+      if (sub_item && GTK_WIDGET_STATE (sub_item) != GTK_STATE_NORMAL)
+	match_data->state = GTK_STATE_SELECTED;
+    }
+}
+
 static void
 draw_box (GtkStyle     *style,
 	  GdkWindow    *window,
@@ -763,6 +787,9 @@ draw_box (GtkStyle     *style,
   match_data.shadow = shadow;
   match_data.state = state;
 
+  if (GTK_IS_MENU_ITEM (widget))
+    maybe_check_submenu_state (GTK_MENU_ITEM (widget), &match_data);
+
   if (!draw_simple_image (style, window, area, widget, &match_data, TRUE,
 			  x, y, width, height)) {
     parent_class->draw_box (style, window, state, shadow, area, widget, detail,
@@ -874,7 +901,15 @@ draw_check (GtkStyle     *style,
   match_data.flags = THEME_MATCH_SHADOW | THEME_MATCH_STATE;
   match_data.shadow = shadow;
   match_data.state = state;
-  
+
+  /* Special casing for GtkCheckButton: We want to set the widget state to
+   * ACTIVE to get the correct graphics used in the RC files. Ideally we'd
+   * use the FOCUS rules, but this is not possible due to technical limitations
+   * in how focus is drawn in sapwood */
+  if (GTK_IS_CHECK_BUTTON (widget) &&
+      GTK_WIDGET_HAS_FOCUS (widget))
+    match_data.state = GTK_STATE_ACTIVE;
+
   if (!draw_simple_image (style, window, area, widget, &match_data, TRUE,
 			  x, y, width, height))
     parent_class->draw_check (style, window, state, shadow, area, widget, detail,
@@ -905,6 +940,14 @@ draw_option (GtkStyle      *style,
   match_data.shadow = shadow;
   match_data.state = state;
   
+  /* Special casing for GtkRadioButton: We want to set the widget state to
+   * ACTIVE to get the correct graphics used in the RC files. Ideally we'd
+   * use the FOCUS rules, but this is not possible due to technical limitations
+   * in how focus is drawn in sapwood */
+  if (GTK_IS_RADIO_BUTTON (widget) &&
+      GTK_WIDGET_HAS_FOCUS (widget))
+    match_data.state = GTK_STATE_ACTIVE;
+
   if (!draw_simple_image (style, window, area, widget, &match_data, TRUE,
 			  x, y, width, height))
     parent_class->draw_option (style, window, state, shadow, area, widget, detail,
@@ -934,7 +977,7 @@ draw_tab (GtkStyle     *style,
   match_data.flags = THEME_MATCH_SHADOW | THEME_MATCH_STATE;
   match_data.shadow = shadow;
   match_data.state = state;
-  
+
   if (!draw_simple_image (style, window, area, widget, &match_data, TRUE,
 			  x, y, width, height))
     parent_class->draw_tab (style, window, state, shadow, area, widget, detail,
